/*
 * Generic event loop functionality.
 *
 * (c) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include <pthread.h>
#include <pthread-l4.h>

/* Event loop abstraction. */

class Event_loop
{
protected:
  /* Private data for the handler. */

  void *_priv;

  /* Thread properties. */

  int _priority;
  pthread_t _pthread;

public:
  /* Initialise the event loop with an event handler function and private data,
     plus thread priority. */

  explicit Event_loop(void *priv, int priority=0x20)
  : _priv(priv), _priority(priority)
  {
  }

  /* Static main loop function for event-handling thread. */

  static void *event_loop(void *data);

  /* Event handler method. */

  virtual void handle() = 0;

  /* Initiation function. */

  virtual void start();
};

#endif
